<?php
///
// Isolde (Revision r168core-r128app)
//
// A third party package for CodeIgniter
//
// Copyright (c) 2013 Kenta Saito (gatedruid.com)
// License       MIT License (See ../isolde_license.txt)
//
?>
<?php
class Crud_core extends CI_Controller {

	public function __construct()
	{
		parent::__construct();

		$this->load->add_package_path(APPPATH.'third_party/isolde_core');
		$this->load->library('form_validation');
		$this->load->library('isolde');
		$this->load->library('image');

		$this->isolde->enable_crud();
	}

	public function index($offset = FALSE)
	{
		$this->session->unset_userdata('offset');
		$sort_columns = FALSE;
		if ($this->input->get('sort'))
		{
			$sort = $this->db->escape_str($this->input->get('sort'));
			$key = preg_replace('/_(asc|desc)$/', '', $sort);
			$dir = (preg_replace('/.*_/', '', $sort) == 'desc' ? 'DESC' : 'ASC');
			$session_sort_columns = $this->session->userdata('sort_columns');
			if ($session_sort_columns === FALSE)
			{
				$session_sort_columns = array();
			}
			if (isset($session_sort_columns[$key]))
			{
				unset($session_sort_columns[$key]);
			}
			$session_sort_columns = array_merge(array($key => $dir), $session_sort_columns);
			$this->session->set_userdata('sort_columns', $session_sort_columns);
		}
		if ($this->input->get('per_page') && in_array($this->input->get('per_page'), $this->config->item('per_page_list')))
		{
			$this->session->set_userdata('per_page', $this->input->get('per_page'));
		}
		if (intval($offset))
		{
			$this->session->set_userdata('offset', $offset);
		}
		if ($this->input->get('clear'))
		{
			$this->session->unset_userdata('sort_columns');
			$this->session->unset_userdata('per_page');
			$this->session->unset_userdata('condition_values');
			if (isset($this->config->item('default_conditions')[$this->isolde->segments->actor][$this->isolde->segments->target]))
			{
				$config_conditions = $this->config->item('default_conditions')[$this->isolde->segments->actor][$this->isolde->segments->target];
				foreach ($config_conditions as $key => $values)
				{
					if ( ! is_array($values))
					{
						$values = array($values);
					}
					foreach ($values as $n => $value)
					{
						if (isset($this->isolde->auth->id))
						{
							$config_conditions["{$key}-{$n}"] = preg_replace('/{\$auth->id}/', $this->isolde->auth->id, $value);
						}
					}
					unset($config_conditions[$key]);
				}
				$this->session->set_userdata('condition_values', serialize((object) $config_conditions));
			}
		}
		if ($this->input->post())
		{
			$condition_values = unserialize($this->session->userdata('condition_values'));
			if ($condition_values === FALSE)
			{
				$condition_values = (object) array();
			}
			foreach ($this->input->post() as $key => $value)
			{
				$condition_values->$key = $value;
			}
			$this->session->set_userdata('condition_values', serialize($condition_values));
		}
		else if ($this->input->get('condition'))
		{
			list($key, $value) = explode('=', $this->input->get('condition'));
			$key .= '-0';
			$condition_values = (object) array();
			$condition_values->$key = $value;
			$this->session->set_userdata('condition_values', serialize($condition_values));
		}
		else
		{
			$condition_values = unserialize($this->session->userdata('condition_values'));
			if ($condition_values === FALSE)
			{
				$condition_values = (object) array();
			}
		}
		if ($this->session->userdata('per_page') === FALSE)
		{
			$this->session->set_userdata('per_page', $this->config->item('per_page'));
		}
		if ($this->session->userdata('offset') === FALSE)
		{
			$this->session->set_userdata('offset', 0);
		}

		$this->isolde->load_model();

// print_r($condition_values); echo '<br />';

		$conditions = array();
		$array = $condition_values;
		if (isset($this->config->item('search')[$this->isolde->segments->actor][$this->isolde->segments->target]))
		{
			$array = $this->config->item('search')[$this->isolde->segments->actor][$this->isolde->segments->target];
			$type_list = $array;
		}
		foreach ($array as $key => $value)
		{
			$key = preg_replace('/-\d+$/', '', $key);
			$a = array('');
			if (isset($type_list[$key]) && is_array($type_list[$key]))
			{
				$a = $type_list[$key];
			}
			foreach ($a as $k => $v)
			{
				$kvk = $key.'-'.$k;
				if ( ! isset($condition_values->$kvk) OR $condition_values->$kvk=== '')
				{
					continue;
				}
				$kvk = $this->db->escape_str($kvk);
				$value = $this->db->escape_str($condition_values->$kvk);
				$type = '=';
				if (isset($type_list[$key]))
				{
					$type = $type_list[$key];
					if (is_array($type))
					{
						$type = $type[$k];
					}
				}
				if ($type == 'like')
				{
					$conditions[] = "{$this->isolde->segments->target}.{$key} like '%{$value}%'";
				}
				else if (in_array($type, array('<', '<=', '>', '>=')))
				{
					$conditions[] = "{$this->isolde->segments->target}.{$key} {$type} '{$value}'";
				}
				else
				{
					$conditions[] = "{$this->isolde->segments->target}.{$key} = '{$value}'";
				}
			}
		}
// print_r($conditions); echo '<br />';

		$fields = $this->crud_model->list_fields();
		$total_rows = $this->crud_model->total_rows(FALSE, $conditions);
		$result = $this->crud_model->index(
			FALSE,
			$conditions,
			$this->session->userdata('sort_columns'),
			$this->session->userdata('offset'),
			$this->session->userdata('per_page')
		);

		$this->isolde->load_view('container', array(
			'segments' => $this->isolde->segments,
			'fields' => $fields,
			'total_rows' => $total_rows,
			'result' => $result,
		));
	}

	public function view($id)
	{
		if (isset($this->config->item('association')['has_many'][$this->isolde->segments->target]))
		{
			$id_escaped = $this->db->escape($id);
			$has_many = array();
			$target_list = $this->config->item('association')['has_many'][$this->isolde->segments->target];
			foreach ($target_list as $target => $key_fields)
			{
				if (
					isset($this->config->item('targets')[$this->isolde->segments->actor]) &&
					(
						! in_array($target, $this->config->item('targets')[$this->isolde->segments->actor]) OR
						! in_array('index', $this->config->item('actions')[$this->isolde->segments->actor][$target])
					)
				)
				{
					continue;
				}

				$this->isolde->load_model($target);

				$segments = (object) array(
					'actor' => $this->isolde->segments->actor,
					'target' => $target,
					'action' => 'index',
				);
				$fields = $this->crud_model->list_fields($target);
				$conditions = array("{$target}.{$key_fields[0]} = {$id_escaped}");
				$total_rows = $this->crud_model->total_rows($target, $conditions);
				$result = $this->crud_model->index(
					$target,
					$conditions,
					FALSE,
					0,
					$this->session->userdata('per_page')
				);
				$has_many[] = (object) array(
					'key_field' => $key_fields[0], 
					'segments' => $segments,
					'fields' => $fields,
					'total_rows' => $total_rows,
					'result' => $result,
				);
			}
		}

		$this->isolde->load_model();

		$fields = $this->crud_model->list_fields();
		$row = $this->crud_model->view($id);
		if ($row === array())
		{
			show_404();
		}
		$this->isolde->load_view('container', array(
			'segments' => $this->isolde->segments,
			'fields' => $fields,
			'row' => $row,
			'has_many' => isset($has_many) ? $has_many : FALSE,
		));
	}

	public function add()
	{
		$this->isolde->load_model();

		$fields = $this->crud_model->list_fields();

		$this->isolde->load_view('container', array(
			'segments' => $this->isolde->segments,
			'fields' => $fields,
		));
	}

	public function edit($id)
	{
		$this->isolde->load_model();

		$row = $this->crud_model->edit($id);
		if ($row === array())
		{
			show_404();
		}
		if ($this->is_not_editable_row($row))
		{
			show_404();
		}

		$fields = $this->crud_model->list_fields();

		$this->isolde->load_view('container', array(
			'segments' => $this->isolde->segments,
			'fields' => $fields,
			'row' => $row,
		));
	}

	public function delete($id)
	{
		$this->isolde->load_model();

		$row = $this->crud_model->delete($id);
		if ($row === array())
		{
			show_404();
		}
		if ($this->is_not_deletable_row($row))
		{
			show_404();
		}

		$fields = $this->crud_model->list_fields();
		$this->isolde->load_view('container', array(
			'segments' => $this->isolde->segments,
			'fields' => $fields,
			'row' => $row,
		));
	}

	public function file($id)
	{
		$this->isolde->load_model();
		$row = $this->crud_model->view($id);
		if ($row === array())
		{
			show_404();
		}
		$field = $this->input->get('field') ? $this->input->get('field') : 'file';
		$contents = $this->input->get('thumb') ? 'thumb' : 'contents';
		header("Content-Disposition: inline; filename=\"{$row->{$field.'_name'}}\"");
		header("Content-Length: {$row->{$field.'_size'}}");
		header("Content-Type: {$row->{$field.'_type'}}");
		echo $row->{$field.'_'.$contents};
	}

	public function affect_add()
	{
		$target = $this->isolde->segments->target;

		$this->isolde->load_model();

		$this->set_form_values($this->input->post());
		$this->validate($target, $this->isolde->segments->action);

		$result = $this->crud_model->affect_add($this->input->post());
		$id = $this->db->insert_id();

		if ($result)
		{
			$row = $this->crud_model->view($id);
			$this->notification($row, $target);
			$this->isolde->append_flash_message(lang('success_add'), 'success');
			redirect($this->input->get('uri'));
		}
		else
		{
			$this->isolde->append_flash_message(lang('error_add'), 'danger');
			redirect(href("{$this->isolde->segments->target}/add/{$this->isolde->segments->id}?uri={$this->input->get('uri')}", TRUE));
		}
	}

	public function affect_edit($id)
	{
		$target = $this->isolde->segments->target;

		$this->isolde->load_model();

		$this->set_form_values($this->input->post());
		$this->validate($target, $this->isolde->segments->action);

		$row = $this->crud_model->edit($id);
		if ($row === array())
		{
			show_404();
		}
		if ($this->is_not_editable_row($row))
		{
			show_404();
		}

		$result = $this->crud_model->affect_edit($id, $this->input->post());

		if ($result)
		{
			$row = $this->crud_model->view($id);
			$this->notification($row, $target);
			$this->isolde->append_flash_message(lang('success_edit'), 'success');
			redirect($this->input->get('uri'));
		}
		else
		{
			$this->isolde->append_flash_message(lang('error_edit'), 'danger');
			redirect(href("{$this->isolde->segments->target}/edit/{$this->isolde->segments->id}?uri={$this->input->get('uri')}", TRUE));
		}
	}

	public function affect_delete($id)
	{
		$this->isolde->load_model();

		$row = $this->crud_model->delete($id);
		if ($row === array())
		{
			show_404();
		}
		if ($this->is_not_deletable_row($row))
		{
			show_404();
		}

		$result = $this->crud_model->affect_delete($id);

		if ($result)
		{
			$row = $this->crud_model->view($id);
			$this->notification($row, $target);
			$this->isolde->append_flash_message(lang('success_delete'), 'success');
			redirect($this->input->get('uri'));
		}
		else
		{
			$this->isolde->append_flash_message(lang('error_delete'), 'danger');
			redirect(href("{$this->isolde->segments->target}/delete/{$this->isolde->segments->id}?uri={$this->input->get('uri')}", TRUE));
		}
	}

	protected function is_not_editable_row($row)
	{
		$not_editable_row = FALSE;

		if (isset($this->isolde->config->editable_rows[$this->isolde->segments->actor][$this->isolde->segments->target]))
		{
			$editable_row = $this->isolde->config->editable_rows[$this->isolde->segments->actor][$this->isolde->segments->target];
			foreach ($editable_row as $key => $value)
			{
				$value = preg_replace('/{\$auth->id}/', $this->isolde->auth->id, $value);
				if ($row->$key !== $value)
				{
					$not_editable_row = TRUE;
					break;
				}
			}
		}

		return $not_editable_row;
	}

	protected function is_not_deletable_row($row)
	{
		$not_deletable_row = FALSE;

		if (isset($this->isolde->config->deletable_rows[$this->isolde->segments->actor][$this->isolde->segments->target]))
		{
			$deletable_row = $this->isolde->config->deletable_rows[$this->isolde->segments->actor][$this->isolde->segments->target];
			foreach ($deletable_row as $key => $value)
			{
				$value = preg_replace('/{\$auth->id}/', $this->isolde->auth->id, $value);
				if ($row->$key !== $value)
				{
					$not_deletable_row = TRUE;
					break;
				}
			}
		}

		return $not_deletable_row;
	}

	protected function set_form_values($values)
	{
		if ($values === FALSE)
		{
			return;
		}
		$form_values = (object) array();
		foreach ($values as $key => $value)
		{
			$form_values->$key = $value;
		}
		$this->session->set_flashdata('form_values', serialize($form_values));
	}

	protected function validate($target, $action)
	{
		$action = preg_replace('/^affect_/', '', $action);
		if (isset($this->isolde->config->validations[$this->isolde->segments->actor][$target][$action]) && count($this->isolde->config->validations[$this->isolde->segments->actor][$target][$action]) > 0)
		{
			$validation_list = $this->isolde->config->validations[$this->isolde->segments->actor][$target][$action];
			$fields = $this->crud_model->list_fields($target);
			foreach ($validation_list as $field => $validations)
			{
				$this->form_validation->set_rules(
					$field,
					$fields->{$field}->COLUMN_COMMENT,
					implode('|', $validations)
				);
			}
			if ($this->form_validation->run() === FALSE)
			{
				$this->isolde->append_flash_message(lang("error_{$action}"), 'danger');
				$this->isolde->append_flash_message(validation_errors(), 'danger');
				redirect(href("{$this->isolde->segments->target}/{$action}/{$this->isolde->segments->id}?uri={$this->input->get('uri')}", TRUE));
			}
		}
	}

	protected function notification($row, $target)
	{
		$action = preg_replace('/^affect_/', '', $this->isolde->segments->action);
		if (isset($this->isolde->config->notification[$this->isolde->segments->actor][$target][$action]) && count($this->isolde->config->notification[$this->isolde->segments->actor][$target][$action]) > 0)
		{
			foreach ($this->isolde->config->notification[$this->isolde->segments->actor][$target][$action] as $to)
			{
				eval('$message = "'.file_get_contents(APPPATH."config/notification/{$this->isolde->segments->actor}_{$target}_{$action}_{$to}.php").'";');
				mail_utf8($row->$to, 'notification', $message);
			}
		}
	}
}
