<?php
///
// Isolde (Revision r168core-r128app)
//
// A third party package for CodeIgniter
//
// Copyright (c) 2013 Kenta Saito (gatedruid.com)
// License       MIT License (See ../isolde_license.txt)
//
?>
<?php
class Crud_core_model extends CI_Model {

	public function __construct($params = array())
	{
		parent::__construct($params);
// $this->output->enable_profiler(TRUE);
	}

	public function list_targets($database = FALSE)
	{
		if ($database === FALSE)
		{
			$database = $this->db->database;
		}
		$database = $this->db->escape_str($database);

		$query =
			'SELECT * FROM information_schema.TABLES '.
			"WHERE TABLE_SCHEMA = '{$this->db->database}' ".
			'ORDER BY TABLE_NAME'
		;
		$result = $this->db->query($query)->result();

		$targets = (object) array();

		foreach ($result as $row)
		{
			if ($row->TABLE_COMMENT == '')
			{
				$row->TABLE_COMMENT = "({$row->TABLE_NAME})";
			}
			$targets->{$row->TABLE_NAME} = $row;
		}

		return $targets;
	}

	public function list_fields($target = FALSE, $database = FALSE)
	{
		if ($target === FALSE)
		{
			$target = $this->isolde->segments->target;
		}

		$fields = (object) array();

		if (isset($this->isolde->config->fields[$this->isolde->segments->actor][$target][$this->isolde->segments->action]))
		{
			$config_fields = $this->isolde->config->fields[$this->isolde->segments->actor][$target][$this->isolde->segments->action];
			foreach ($config_fields as $key)
			{
				$raw_key = $key;
				$key = preg_replace('/.* AS /i', '', $key);
				$fields->$raw_key = (object) array(
					'COLUMN_NAME' => $key,
					'COLUMN_COMMENT' => $key,
				);
			}
		}

		$raw_target = $target;
		$target = $this->db->escape($target);

		if ($database === FALSE)
		{
			$database = $this->db->database;
		}
		$database = $this->db->escape_str($database);

		$query =
			'SELECT * FROM information_schema.COLUMNS '.
			"WHERE TABLE_SCHEMA = '{$this->db->database}' AND TABLE_NAME = {$target} ".
			'ORDER BY ORDINAL_POSITION'
		;
		$result = $this->db->query($query)->result();

		foreach ($result as $row)
		{
			if ($row->COLUMN_COMMENT == '')
			{
				$row->COLUMN_COMMENT = "({$row->COLUMN_NAME})";
			}
			if ($row->COLUMN_NAME == 'password')
			{
				$row->DATA_TYPE = 'password';
			}
			if (isset($this->isolde->config->data_type[$raw_target][$row->COLUMN_NAME]))
			{
				$row->DATA_TYPE = $this->isolde->config->data_type[$raw_target][$row->COLUMN_NAME];
			}
			$fields->{$row->COLUMN_NAME} = $row;
		}

		return $fields;
	}

	public function primary($target = FALSE, $database = FALSE)
	{
		if ($target === FALSE)
		{
			$target = $this->isolde->segments->target;
		}
		$target = $this->db->escape($target);

		if ($database === FALSE)
		{
			$database = $this->db->database;
		}
		$database = $this->db->escape_str($database);

		$query =
			'SELECT COLUMN_NAME FROM information_schema.COLUMNS '.
			"WHERE TABLE_SCHEMA = '{$this->db->database}' AND TABLE_NAME = {$target} AND COLUMN_KEY = 'PRI' ".
			'ORDER BY ORDINAL_POSITION'
		;
		$primary = $this->db->query($query)->row()->COLUMN_NAME;

		return $primary;
	}

	public function total_rows($target = FALSE, $conditions = FALSE)
	{
		if ($target === FALSE)
		{
			$target = $this->isolde->segments->target;
		}
		$target = $this->db->escape_str($target);

		$query = "SELECT COUNT(*) AS total_rows FROM {$target}";

		if (isset($this->isolde->config->join[$this->isolde->segments->actor][$target]))
		{
			$query .= ' '.$this->isolde->config->join[$this->isolde->segments->actor][$target];
		}

		$conditions = $this->merge_config_conditions($conditions, $target);
		if ($conditions)
		{
			$query .= " WHERE ".implode(' AND ', $conditions);
		}
// echo $query.'<br />';
		$total_rows = $this->db->query($query)->row()->total_rows;

		return $total_rows;
	}

	public function index($target = FALSE, $conditions = FALSE, $sort_columns = FALSE, $offset = FALSE, $row_count = FALSE)
	{
		if ($target === FALSE)
		{
			$target = $this->isolde->segments->target;
		}
		$target = $this->db->escape_str($target);
		$primary = $this->primary($target);
		if ($sort_columns === FALSE)
		{
			if (isset($this->config->item('sort')[$target]))
			{
				$sort_columns = $this->config->item('sort')[$target];
			}
			else
			{
				$sort_columns = array($primary => 'ASC');
			}
		}
		$sort_columns_array = array();
		foreach ($sort_columns as $key => $dir)
		{
			$key = $this->db->escape_str($key);
			$sort_columns_array[] = "{$key} {$dir}";
		}

		$columns = $this->get_columns($target);

		$query = "SELECT ".implode(', ', $columns)." FROM {$target}";

		if (isset($this->isolde->config->join[$this->isolde->segments->actor][$target]))
		{
			$query .= ' '.$this->isolde->config->join[$this->isolde->segments->actor][$target];
		}

		$conditions = $this->merge_config_conditions($conditions, $target);
		if ($conditions)
		{
			$query .= " WHERE ".implode(' AND ', $conditions);
		}

		if (isset($this->isolde->config->group_by[$this->isolde->segments->actor][$target]))
		{
			$query .= ' GROUP BY '.$this->isolde->config->group_by[$this->isolde->segments->actor][$target];
		}

		$query .= " ORDER BY {$target}.".implode(", {$target}.", $sort_columns_array);

		if ($offset !== FALSE && $row_count !== FALSE)
		{
			$offset = $this->db->escape_str($offset);
			$row_count = $this->db->escape_str($row_count);
			$query .= " LIMIT {$offset}, {$row_count}";
		}
// echo $query;
		$result = $this->db->query($query)->result();

		return $result;
	}

	public function view($id, $target = FALSE)
	{
		if ($target === FALSE)
		{
			$target = $this->isolde->segments->target;
		}
		$target = $this->db->escape_str($target);
		$primary = $this->primary();
		$id = $this->db->escape($id);

		$conditions = array("{$target}.{$primary} = {$id}");
		$conditions = $this->merge_config_conditions($conditions, $target);

		$columns = $this->get_columns($target);

		$query =
			"SELECT ".implode(', ', $columns)." FROM {$target}"
		;

		if (isset($this->isolde->config->join[$this->isolde->segments->actor][$target]))
		{
			$query .= ' '.$this->isolde->config->join[$this->isolde->segments->actor][$target];
		}

		$query .=
			" WHERE ".implode(' AND ', $conditions)
		;

		$row = $this->db->query($query)->row();

		return $row;
	}

	public function edit($id, $target = FALSE)
	{
		if ($target === FALSE)
		{
			$target = $this->isolde->segments->target;
		}
		$target = $this->db->escape_str($target);
		$primary = $this->primary();
		$id = $this->db->escape($id);

		$conditions = array("{$target}.{$primary} = {$id}");
		$conditions = $this->merge_config_conditions($conditions, $target);

		$columns = $this->get_columns($target);

		$query =
			"SELECT ".implode(', ', $columns)." FROM {$target}"
		;

		if (isset($this->isolde->config->join[$this->isolde->segments->actor][$target]))
		{
			$query .= ' '.$this->isolde->config->join[$this->isolde->segments->actor][$target];
		}

		$query .=
			" WHERE ".implode(' AND ', $conditions)
		;

		$row = $this->db->query($query)->row();

		return $row;
	}


	public function delete($id, $target = FALSE)
	{
		if ($target === FALSE)
		{
			$target = $this->isolde->segments->target;
		}
		$target = $this->db->escape_str($target);
		$primary = $this->primary();
		$id = $this->db->escape($id);

		$conditions = array("{$target}.{$primary} = {$id}");
		$conditions = $this->merge_config_conditions($conditions, $target);

		$columns = $this->get_columns($target);

		$query =
			"SELECT ".implode(', ', $columns)." FROM {$target}"
		;

		if (isset($this->isolde->config->join[$this->isolde->segments->actor][$target]))
		{
			$query .= ' '.$this->isolde->config->join[$this->isolde->segments->actor][$target];
		}

		$query .=
			" WHERE ".implode(' AND ', $conditions)
		;

		$row = $this->db->query($query)->row();

		return $row;
	}

	public function affect_add($values, $target = FALSE)
	{
		$this->store_file($values);
		if ($target === FALSE)
		{
			$target = $this->isolde->segments->target;
		}
		$target = $this->db->escape_str($target);
		foreach ($values as $key => $value)
		{
			$key = $this->db->escape_str($key);
			if ($key == 'password')
			{
				$value = $this->encrypt->sha1($value.$this->config->item('encryption_key'));
				$value = "'{$value}'";
			}
			else
			{
				$value = $this->db->escape($value);
			}
			$s_keys[] = $key;
			$s_values[] = $value;
		}
		$query =
			"INSERT INTO {$target} (".implode(', ', $s_keys).") VALUES (".implode(', ', $s_values).")"
		;
		$result = $this->db->query($query);

		return $result;
	}

	public function affect_edit($id, $values, $target = FALSE)
	{
		$this->store_file($values);
		if ($target === FALSE)
		{
			$target = $this->isolde->segments->target;
		}
		$target = $this->db->escape_str($target);
		foreach ($values as $key => $value)
		{
			$key = $this->db->escape_str($key);
			if ($key == 'password')
			{
				if ($value === '')
				{
					continue;
				}
				$value = $this->encrypt->sha1($value.$this->config->item('encryption_key'));
				$value = "'{$value}'";
			}
			else
			{
				$value = $this->db->escape($value);
			}
			$key_values[] = "{$key} = {$value}";
		}
		$primary = $this->primary();
		$id = $this->db->escape($id);

		$conditions = array("{$primary} = {$id}");

		$query =
			"UPDATE {$target} SET ".implode(', ', $key_values).' '.
			"WHERE ".implode(' AND ', $conditions)
		;
		$result = $this->db->query($query);

		return $result;
	}

	public function affect_delete($id, $target = FALSE)
	{
		if ($target === FALSE)
		{
			$target = $this->isolde->segments->target;
		}
		$target = $this->db->escape_str($target);
		$primary = $this->primary();
		$id = $this->db->escape($id);

		$conditions = array("{$primary} = {$id}");

		$query =
			"DELETE FROM {$target} ".
			"WHERE ".implode(' AND ', $conditions)
		;
		$result = $this->db->query($query);

		return $result;
	}

	protected function merge_config_conditions($conditions, $target)
	{
		if (isset($this->isolde->config->conditions[$this->isolde->segments->actor][$target]))
		{
			$config_conditions = $this->isolde->config->conditions[$this->isolde->segments->actor][$target];
			foreach ($config_conditions as $key => $value)
			{
				if (isset($this->isolde->auth->id))
				{
					$config_conditions[$key] = preg_replace('/{\$auth->id}/', $this->isolde->auth->id, $value);
				}
			}
			if ($conditions)
			{
				$conditions = array_merge($conditions, $config_conditions);
			}
			else
			{
				$conditions = $config_conditions;
			}
		}

		return $conditions;
	}

	protected function get_columns($target)
	{
		$fields = $this->list_fields($target);
		$columns = array_keys((array) $fields);
		foreach ($columns as $key => $value)
		{
			if (preg_match('/ AS /i', $value))
			{
				$columns[$key] = $value;
			}
			else
			{
				$columns[$key] = "{$target}.{$value}";
			}
		}

		return $columns;
	}

	protected function store_file(&$values)
	{
		if (isset($_FILES))
		{
			foreach ($_FILES as $key => $file)
			{
				if ($file['error'] !== 0)
				{
					return FALSE;
				}
				$values[$key] = $file['name'];
				$values[preg_replace('/name$/', 'type', $key)] = $file['type'];
				$values[preg_replace('/name$/', 'size', $key)] = $file['size'];
				$values[preg_replace('/name$/', 'contents', $key)] = file_get_contents($file['tmp_name']);
				if (in_array($file['type'], array('image/jpeg', 'image/png', 'image/gif')))
				{
					$this->image->set_imagedata(file_get_contents($file['tmp_name']));
					$values[preg_replace('/name$/', 'thumb', $key)] = $this->image->trim(
						$this->isolde->config->thumbnail['width'],
						$this->isolde->config->thumbnail['height']
					)->resample(TRUE);
				}
			}
		}
	}
}
